Multiscroll.prototype.activate = Multiscroll.defaultPrefs.activate;
Multiscroll.prototype.cancelKey = Multiscroll.defaultPrefs.cancelKey;
Multiscroll.prototype.exitAction = Multiscroll.defaultPrefs.exitAction;

Multiscroll.Event = function(parent, settings) {
	this.parent = parent;
	this.activate = settings.activate;
	this.cancelKey = settings.cancelKey;
	this.exitAction = settings.exitAction;
};
Multiscroll.Event.prototype = {
	activateButtonDown : false,
	snapOn : false,
	clientX : 0,
	clientY : 0,
	savedX : 0,
	savedY : 0,
	browserClientX : 0,
	browserClientY : 0,
	startX : 0,
	startY : 0,
	goodToActivate : function(evt) {
		return parseInt(evt.button) == this.activate.button
			 && evt.shiftKey == this.activate.shift
			 && evt.ctrlKey == this.activate.ctrl
			 && evt.altKey == this.activate.alt
			 && evt.metaKey == this.activate.meta;
	},
	startInitial : function(evt) { with (this) {
		snapOn = true;
		parent.action.initial.start(evt);
		var frame = evt.originalTarget.ownerDocument.defaultView;
		savedX = frame.pageXOffset;
		savedY = frame.pageYOffset;
		startX = evt.clientX;
		startY = evt.clientY;
	} },
	click : function(evt) { with (multiscroll.event) {
		var button = parseInt(evt.button);
		if (goodToActivate(evt) && !parent.action.initial.active
				 && evt.detail == activate.action)
			startInitial(evt);
	} },
	mouseDown : function(evt) { with (multiscroll.event) {
		var button = parseInt(evt.button);
		var action = parent.actionByButton[button];
	
		if (button == activate.button)
			activateButtonDown = true;
		if (activate.action == Multiscroll.mouse.action.mousedown
				 && goodToActivate(evt)
				 && !parent.action.initial.active) {
			if (!isLink(evt.originalTarget)
					 && !isChrome(evt.target)
					 && !isTabStrip(evt.originalTarget))
				startInitial(evt);
		} else if (parent.action.initial.active && !(action.tool instanceof Multiscroll.Tool.None)) {
			evt.preventDefault();
			action.active = true;
			parent.action.initial.stop(evt);
			action.start(evt);
		} else if (parent.action.initial.done) {
			parent.action.initial.stop(evt);
		}
	} },
	mouseUp : function(evt) { with (multiscroll.event) {
		var button = parseInt(evt.button);
		var action = parent.actionByButton[button];

		if (button == activate.button)
			activateButtonDown = false;
		if (action.active && !(action.tool instanceof Multiscroll.Tool.None)
				 && !parent.action.initial.active)
			action.stop(evt);
		if (!snapOn || exitAction == Multiscroll.mouse.action.release)
			parent.action.initial.stop(evt);
	} },
	mouseMove : function(evt) { with (multiscroll.event) {
		if (!isChrome(evt.target) && !isTabStrip(evt.originalTarget)) {
			clientX = evt.clientX;
			clientY = evt.clientY;
			browserClientX = evt.screenX - window.screenX - clientX;
			browserClientY = evt.screenY - window.screenY - clientY;
		} else {
			clientX = evt.screenX - window.screenX - browserClientX;
			clientY = evt.screenY - window.screenY - browserClientY;
		}	// we update clientX and clientY for scroll tool
	
		if (activateButtonDown
				 && exitAction == Multiscroll.mouse.action.drag
				 && !snapOn)
			parent.action.initial.stop(evt);
		
		if (parent.getActiveTool().execute)
			parent.getActiveTool().execute(evt);

		if (parent.action.initial.active) {
			var dx = Math.abs(clientX - startX);
			var dy = Math.abs(clientY - startY);
	
			if (dx > dragRadius || dy > dragRadius)
				snapOn = false;
		}
	} },
	resize : function(evt) { with (multiscroll.event) {
		if (panThumbnailPreload)
			for (action in parent.action)
				if (parent.action[action].tool instanceof Multiscroll.Pan
						&& parent.action[action].active)
					setTimeout(parent.action[action].createThumbnailInAdvance, 1, evt);
	} },
	load : function(evt) { with (multiscroll.event) {
		if (panThumbnailPreload)
			for (action in parent.action)
				if (parent.action[action].tool instanceof Multiscroll.Pan
						&& parent.action[action].active)
					setTimeout(parent.action[action].createThumbnailInAdvance, 1, evt);
	} },
	keyPress : function(evt) { with (multiscroll.event) {
		if (evt.keyCode == parent.cancelKey && !parent.none.active) {
			parent.getActiveTool().client.frame.scrollTo(savedX, savedY);
			// MUST be in this order otherwise we get redraw problems
			parent.none.start(evt);
		}
	} },
	isLink : function(node) {
		if (!node)
			return false;
		else if (node.nodeName == "A" || node.nodeName == "INPUT"
			 || node.nodeName == "TEXTAREA" || node.nodeName == "AREA")
			return true;
		else
			return this.isLink(node.parentNode);
	},
	isChrome : function(target) {
	        // Is there a better way to do this?
	        if (target.id != "content")
	                return true;
	        else
	                return false;
	},
	isTabStrip : function(target) {
		// There must be a better way!
		if (target.nodeName == "tab" || target.nodeName.slice(0, 4) == "xul:" )
			return true;
		else
			return false;
	},
	callis : function() {
		return this.isChrome({id : "content"});
	}
};
